%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% -*- Mode: Latex -*- %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 
%% 
%% QNX-inote.cls -- 
%% 
%% Author           : anil k goel
%% Created On       : Oct  6 14:27:13 1995
%% Last Modified By : Bobby Chawla
%% Last Modified On : Fri July  5 12:14:45 1996
%% Update Count     : 1400
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% QNX-book class for the use of QNX Software Systems.
% Based on specifications provided by {bobbyc,chrish}@qnx.com

\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{QNX-inote}[1995/10/07]
\typeout{QNX-inote style v1.1 July 1996}



% Printing on 8.5x11 paper  - don't change paper dimensions
% These must be changed at the beginning before tha class is loaded.
%\setlength{\paperheight}{9in}
%\setlength{\paperwidth}{7.25in}
\setlength{\paperheight}{11in}
\setlength{\paperwidth}{8.5in}


% Load the standard book document class
\DeclareOption*{\PassOptionsToClass{\CurrentOption}{book}}
\ProcessOptions

\LoadClass[11pt,twoside]{book}


%%%%
%
% Load packages 
%
%%%%
\RequirePackage{graphics}[1994/12/15]	% graphics commands
\RequirePackage{ltablex}[1995/11/06]% for the conventions section
\RequirePackage{multicol}[1994/10/02]	% multiple column sections
\RequirePackage{alltt}[1995/05/18]
		% TODO: fix this for marginNotes
\RequirePackage{calc}	[1995/04/10]	% do math in the LaTeX
\RequirePackage{times}[1995/08/16]
\RequirePackage{helvet}[1994/06/16]
\RequirePackage{pifont}[1995/08/16]	% for accessing dingbats [chrish]
\RequirePackage{fancyheadings}

\RequirePackage{setspace}             % for inter-line spacing in X columns
\RequirePackage{ifthen}
\RequirePackage{colordvi}


%%%%
%
%	Two side vs. one side printing
%
% The printing is to be done two sided but the desired location of side margins
% is always on the left. This is contrary to the LaTeX's notion of twoside in
% which the side margins are printed on the outside, i.e., on the left side of
% even and the right side of odd pages. So, it is going to be problematic to
% use the LaTeX two side option. Not using twoside, however, causes the headers/
% footers and side margins to be messed up -- we need more margin on the inside
% than on the outside.
%
% We fix this by redefining \@addmarginpar
%
%   (from refman.dtx)
%
%%%%

\def\@addmarginpar{\@next\@marbox\@currlist{\@cons\@freelist\@marbox
    \@cons\@freelist\@currbox}\@latexbug\@tempcnta\@ne
    \if@twocolumn
       \if@firstcolumn \@tempcnta\m@ne \fi
    \else
       \@tempcnta\m@ne
    \fi
    \ifnum\@tempcnta <\z@  \global\setbox\@marbox\box\@currbox \fi
    \@tempdima\@mparbottom
    \advance\@tempdima -\@pageht
    \advance\@tempdima\ht\@marbox
    \ifdim\@tempdima >\z@
      \@@warning{Marginpar on page \thepage\space moved}%
    \else
      \@tempdima\z@
    \fi
    \global\@mparbottom\@pageht
    \global\advance\@mparbottom\@tempdima
    \global\advance\@mparbottom\dp\@marbox
    \global\advance\@mparbottom\marginparpush
    \advance\@tempdima -\ht\@marbox
    \global\setbox \@marbox
                   \vbox {\vskip \@tempdima \box \@marbox}%
    \global \ht\@marbox \z@
    \global \dp\@marbox \z@
    \kern -\@pagedp
    \nointerlineskip
    \hbox to\columnwidth
      {\ifnum \@tempcnta >\z@
          \hskip\columnwidth \hskip\marginparsep
       \else
          \hskip -\marginparsep \hskip -\marginparwidth
      \fi
      \box\@marbox \hss}%
    \nointerlineskip
    \hbox{\vrule \@height\z@ \@width\z@ \@depth\@pagedp}}

%%%%
%
% Page Layout
%
% Bobby C. sez:
% Dimensions:  Centered on an 8.5x11 page with crop marks
% 
%             Page is 7.25" wide x 9" high
%             1/2" border outside
%             3/4" border inside
%             3/8" top between heading and top-of-page
%             1/2" between top-ruling-line and top-of-page
%             1-1/2"  scholar's margins
%             1/4"    gap between scholar's margins and text.
%             4-1/4"  body (if my math is correct :-)
%
%%%%

% HORIZONTAL DIMENSIONS

% Width of text (includes margins and separation thereof):
\setlength{\textwidth}{5.25in}


% set horizontal offset -1in to take off the default 1in on left
%\setlength{\hoffset}{-1in}
% changed to centre page [chrish]
\setlength{\hoffset}{-0.875in}

% on even pages, the margin on left = outside border = 0.5in
\setlength{\evensidemargin}{.5in}

% On odd pages, the margin on left = inside border = 0.75in
\setlength{\oddsidemargin}{0.5in}

% Marginal notes:
\setlength{\marginparwidth}{1.5in}     % Width of marginal notes.
\setlength{\marginparsep}{0.25in}      % space after marginal note

% some internal stuff
\newlength{\marginpartotalwidth}
\setlength{\marginpartotalwidth}{\marginparwidth}
\addtolength{\marginpartotalwidth}{\marginparsep}

% sanity check: \oddsidemargin = 
                % \paperwidth - \evensidemargin - \textwidth - \marginparwidth - \marginparsep
%	 0.75 = 7.25 - .5 - 4.25 - 1.5 - .25 = 0.75  => good
%

% except that \addmarginpar seems to mess with our left margins. Fix:
\addtolength{\evensidemargin}{\marginpartotalwidth}
\addtolength{\oddsidemargin}{\marginpartotalwidth}

% VERTICAL DIMENSIONS

% take away the default 1in left on top by latex
% changed to centre page [chrish] (was \setlength{\voffset}{-1in})
% \setlength{\voffset}{0in}
%
% Seems we need to add another .75inch to get it where we want
% (can't explain why)   [bobbyc]
%\setlength{\voffset}{.75in - 2pt}
\setlength{\voffset}{0pt}


% need 3/8" space between top of page and header
% \setlength{\topmargin}{0.375in}
% modified to 1/4 to match ``System Architechture'' book [bobbyc]
\setlength{\topmargin}{-0.5in}


% and, a 1/2" vertical space after the header ruling
%\setlength{\headsep}{0.5in}

% A headheight of .19 (~14pt) ought to be good for a 9pt header and a ruling
%\setlength{\headheight}{.19in} 

% Total footer space - same as header + headersep
\setlength{\footskip}{.69in}

% Height of text:
% including footnotes and figures,
% Setting \textheight to 6.87in will leave .375in at the bottom of page
% \setlength{\textheight}{6.87in}  
% Tweak a bit, make it look like the S.A. manual
\setlength{\textheight}{9in}  


%%%%%
% Subject: Re: The \Purpose command
%
%   From: "Bobby Chawla" <bobbyc@qnx.com>
%   Date: Thu, 26 Oct 1995 18:42:55 -0400 (edt)
%   
%   Hi Anil,
%   
%      I was looking at the code, and wondering...
%   If I change the fontstyle/size of purpose, how much
%   troule would that be???   
%
% You probably won't have to change anything if the font size doesn't
% change drasticallly. If you do need to change, the code that would
% need tweaking is the definition of @fancyhead; you would likely only
% have to adjust the three vskip amounts in @fancyhead so that the ruler
% stays at a fixed location...
%
%   Hummm,  I see \addtolength{\headheight} and {\headsep}
%      are hard coded, that I'd have to modify, what else?
%
% I just doubled the \headheight and reduced \headsep with a
% corresponding amount. As long as the font you use for purpose is not
% bigger than about 12pt, you should be okay.... If you end up
% increasing \headheight drastically, you should probably leave 
% \headsep alone and reduce \textheight instead but that is a matter of
% style and you will have to experiment a bit...
%
% -anil
%%%%%

% to accomodate \purposetext
\addtolength{\headheight}{.25in}
\addtolength{\headsep}{-.25in}


%%
%  Re-format things to print out the index using the full
%  width of the page...
%
%  NOTE:  Starts a new page...
%
%  BobbyC
%
%%
\def\startindex{%
\newpage%

% Width of index text
\setlength{\textwidth}{6in}

% set horizontal offset to centre page [chrish]
\setlength{\hoffset}{-0.375in}

% on even pages, the margin on left = outside border = 0.5in
\setlength{\evensidemargin}{.5in}

% On odd pages, the margin on left = inside border = 0.75in
\setlength{\oddsidemargin}{0.75in}

% Marginal notes are no-longer used...:
\setlength{\marginparwidth}{0in}     % Width of marginal notes.
\setlength{\marginparsep}{0in}      % space after marginal note

% some internal stuff
\setlength{\marginpartotalwidth}{\marginparwidth}
\addtolength{\marginpartotalwidth}{\marginparsep}

% Again, the \addmarginpar seems to mess with our left margins.
\addtolength{\evensidemargin}{\marginpartotalwidth}
\addtolength{\oddsidemargin}{\marginpartotalwidth}

\onecolumn
}

%%
%
%	Add a couple of commands for shrinking our graphics if necessary.
%	[chrish]
%
%	\resizedGfx{filename}
%		include a graphic in the text area, and scale it to the width
%		of the column if it's too large
%
%	\resizedMarginGfx{filename}
%
%	TODO: we need one for marginNote graphics, too.
%
%%

% check if there is enough space on this page to hold
% graphics image #1. If not go to next page...
\newdimen\tempakg
\newcommand{\checkMinFree}[1]{
	\setbox\tmpbox=\hbox{\includegraphics{#1}}%
  	\tempakg=\pagegoal
  	\advance\tempakg-\pagetotal
  	\ifdim\tempakg<\ht\tmpbox\newpage\fi
}

	
\newsavebox{\tmpbox}
\newsavebox{\rtmpbox}
\newsavebox{\rrtmpbox}
\newlength{\tmpboxspan}
\newcommand{\sizedGfx}[2][t]{\noindent
\setbox\rrtmpbox=\hbox{\includegraphics{#2}}%
\setlength{\tmpboxspan}{\wd\rrtmpbox}%
\setlength{\tmpboxspan}{\tmpboxspan * 9 / 11}%
\setbox\tmpbox=\hbox{\resizebox{\tmpboxspan}{!}{\box\rrtmpbox}}%
\ifdim\wd\tmpbox>\linewidth%
    \setbox\rtmpbox=\hbox{\resizebox{\linewidth}{!}{\box\tmpbox}}%
    \box\rtmpbox%
\else%
    \box\tmpbox%
\fi%
}

\newcommand{\resizedGfx}[2][t]{\noindent
   \setbox\tmpbox=\hbox{\includegraphics{#2}}%
   \ifdim\wd\tmpbox>\linewidth%
       \setbox\rtmpbox=\hbox{\resizebox{\linewidth}{!}{\box\tmpbox}}%
       \box\rtmpbox%
   \else%
       \raisebox{-\ht\tmpbox}{\includegraphics{#2}}%
   \fi%
}

%
% sets the X size of a given figure, and then
% also checks if it fits on the page.
% [bobbyc]   Nov 10, 1995
%
\newsavebox{\Xtmpbox}
\def\XsizedGfx#1#2{\noindent
\setbox\tmpbox=\hbox{\includegraphics{#1}}%
\setbox\Xtmpbox=\hbox{\resizebox{#2}{!}{\box\tmpbox}}
\ifdim\wd\Xtmpbox>\linewidth%
    \setbox\rtmpbox=\hbox{\resizebox{\linewidth}{!}{\box\Xtmpbox}}
    \box\rtmpbox
\else
    \box\Xtmpbox
\fi
}


\newsavebox{\mtmpbox}
\newsavebox{\mrtmpbox}
\newcommand{\resizedMarginGfx}[2][t]{\noindent
\setbox\mtmpbox=\hbox{\includegraphics{#2}}%
\ifdim\wd\mtmpbox>\marginparwidth%
    \setbox\mrtmpbox=\hbox{\resizebox{\marginparwidth}{!}{\box\mtmpbox}}
    \box\mrtmpbox
\else
  %\tempakg-#1\ht\mtmpbox
  %\advance\tempakg\baselineskip
  %%\advance\tempakg 
  \parbox[t]{\marginparwidth}{\raggedleft \includegraphics{#2}}
\fi}

%%%
%
%  Put graphics in the margin, but allow the ability to position
%  it at the <t>op <b>ottom or <c>enter.
%
%  By akgoel (Dec 1995)
%
%%%
\newcommand{\qresizedMarginGfx}[2][t]{\noindent 
\setbox\mtmpbox=\hbox{\includegraphics{#2}}%
\ifdim\wd\mtmpbox>\marginparwidth%
    \setbox\mtmpbox=\hbox{\resizebox{\marginparwidth}{!}{\box\mtmpbox}}
    \box\mtmpbox
\else
  %\tempakg-#1\ht\mtmpbox
  %\advance\tempakg\baselineskip
  %%\advance\tempakg
  %\vskip \baselineskip
  % {\raggedleft \box\mtmpbox}
  \margin@note{\leavevmode \vskip -0.5\baselineskip\parbox[t]{\marginparwidth}{\raggedleft \copy\mtmpbox}}
  \leavevmode\vskip 0.3\baselineskip
  \ifthenelse{\equal{#1}{t}}{\vskip  0.5\baselineskip\vskip -\ht\mtmpbox}{ } 
  \ifthenelse{\equal{#1}{b}}{\vskip 0pt}{ } 
  \ifthenelse{\equal{#1}{c}}{\vskip  0.5\baselineskip\vskip -0.5\ht\mtmpbox}{ } 
\fi}

%%%
%  And for sizing things in the margin...
%
%  BobbyC  (Nov 28, 1995)
%%%
\def\XsizedMarginGfx#1#2{\noindent
\setbox\mtmpbox=\hbox{\includegraphics{#1}}%
\setbox\Xtmpbox=\hbox{\resizebox{#2}{!}{\box\mtmpbox}}
\ifdim\wd\Xtmpbox>\marginparwidth%
    \setbox\mrtmpbox=\hbox{\resizebox{\marginparwidth}{!}{\box\Xtmpbox}}
    \box\mrtmpbox
\else
    \box\Xtmpbox
\fi}


%%%%
%
%  Define Helvetica Narrow...
%
%%%%
\newcommand{\sfndefault}{phv}
\DeclareRobustCommand\sfnfamily
        {\not@math@alphabet\sfnfamily\mathsf
         \fontfamily\sfndefault\fontseries{bc}\selectfont}


%%%%
%
% Fonts
%
% from BobbyC:
% Fonts:            Text Body:  11pt Times Roman
%              Section title:  24pt Helvetica Bold
%           subsection title:  12pt Helvetica Bold
%        subsubsection title:   8pt Helvetical
%               Page numbers:  11pt Helvetica bold
%                   Captions:   9pt Helvetica italic 
%         Captions in margin:   8pt Helvetica
%               Code samples:   8pt Courier bold
%            Headers/Footers:   9pt Helvetica      <-- No italics, changed from
%                                                              current version
%%%%



% the times package sets the default font to be Times Roman and
%	\rmfamily -> Times Roman
%	\sffamily -> Helvetica
%	\ttfamily -> Courier
% we have already chosen 11pt....

% set up fonts for use later
\newcommand{\ChFont}{\sffamily\bfseries}
\newcommand{\SFont}{\sffamily\bfseries}
\newcommand{\SSFont}{\sffamily\bfseries}
\newcommand{\SSSFont}{\sffamily\bfseries}
\newcommand{\PageFont}{\sffamily\itshape}
\newcommand{\CapFont}{\sffamily\slshape}
\newcommand{\MarginCapFont}{\sffamily}
\newcommand{\CodeFont}{\ttfamily\bfseries}
\newcommand{\MainHeadFont}{\sffamily}
\newcommand{\PseudoHeadFont}{\sffamily\bfseries}
\newcommand{\PurposeFont}{\normalfont\rmfamily\itshape}
\newcommand{\FootFont}{\sffamily}
\newcommand{\TOCHeadFont}{\sffamily\bfseries}
\newcommand{\TOCFont}{\normalfont}

% chrish added these so his brain wouldn't explode
\newcommand{\TitleMainFont}{\sffamily\bfseries}
\newcommand{\TitleSubFont}{\sffamily\bfseries\itshape}
\newcommand{\TitleCmtFont}{\normalfont\rmfamily\itshape}

\newcommand{\ChSize}{\@setfontsize\ChSize{18pt}{21pt}}
\newcommand{\Ssize}{\@setfontsize\Ssize{18pt}{21pt}}
\newcommand{\SSsize}{\@setfontsize\SSsize{12pt}{14pt}}
\newcommand{\SSSsize}{\@setfontsize\SSSsize{9pt}{11pt}}
\newcommand{\PageSize}{\@setfontsize\PageSize{11pt}{13pt}}
\newcommand{\CapSize}{\@setfontsize\CapSize{9pt}{10pt}}
\newcommand{\MarginCapSize}{\@setfontsize\MarginCapSize{8pt}{10pt}}
\newcommand{\CodeSize}{\@setfontsize\CodeSize{8pt}{10pt}}
% Increase Headsize by 2pt.
\newcommand{\MainHeadSize}{\@setfontsize\MainHeadSize{11pt}{13pt}}
\newcommand{\PseudoHeadSize}{\@setfontsize\PseudoHeadSize{16pt}{18pt}}
\newcommand{\PurposeSize}{\@setfontsize\PurposeHeadSize{11pt}{13pt}}
\newcommand{\FootSize}{\@setfontsize\FootSize{9pt}{11pt}}
\newcommand{\TOCHeadSize}{\@setfontsize\TOCHeadSize{14pt}{16pt}}
\newcommand{\TOCSize}{\@setfontsize\TOCSize{11pt}{13pt}}
\newcommand{\TitleMainSize}{\@setfontsize\ChSize{24pt}{26pt}}
\newcommand{\TitleSubSize}{\@setfontsize\ChSize{18pt}{20pt}}
\newcommand{\TitleCmtSize}{\@setfontsize\ChSize{11pt}{13pt}}
\newcommand{\ttSmallSize}{\@setfontsize\ttsize{6.35pt}{8.35pt}}
\newcommand{\ttMediumSize}{\@setfontsize\ttsize{8pt}{10pt}}
\newcommand{\ttLargeSize}{\@setfontsize\ttsize{10pt}{12pt}}

%%%%
%
% Cause LaTeX to SHUT UP about all the wide/un-wide stuff.
%
%%%%
\tolerance=6000

%%%%
%
% Define spacing between table of contents entry 
% and their page numbers.
%
%%%%
\renewcommand\@pnumwidth{2.5em}

%%%%
%
% Make fancyheadings even fancier...
%
%    Two different types of headings, one for regular sections
%    and one for "man pages" or pseudosections
%
%%%%

\newcommand\MainHead{
  \global\let\HeadFont=\MainHeadFont
  \global\let\HeadSize=\MainHeadSize
  \global\def\PseudoHeadColor{}
}

\MainHead

\newcommand\PseudoHead{
  \global\let\HeadFont=\PseudoHeadFont
  \global\let\HeadSize=\PseudoHeadSize
  \global\def\PseudoHeadColor{}
}

%%%%
%
% Fancyheadings - for headers and footers
%
%%%%


% use fancyplain page style so that first page of chapters can have different
% headers/footers.
\pagestyle{fancyplain}

\headrulewidth 0pt
\newif\ifPurposeText@
\PurposeText@false

\def\purposetext{}
\newcommand\Purpose[1]{
  \ifx\@empty\purposetext\global\def\purposetext{\PurposeFont \PurposeSize #1}\fi
  \PurposeText@true
}

% we want the header/footers to cover the margins
\addtolength{\headwidth}{\marginpartotalwidth}

\def\@fancyhead#1#2#3#4#5{\ifodd\count\z@\hspace{-\marginpartotalwidth}\fi #1\hbox to\headwidth{\vbox{\hbox
{\rlap{\parbox[b]{\headwidth}{\raggedright \PseudoHeadColor{#2}\strut}}\hfill
\parbox[b]{\headwidth}{\centering#3\strut}\hfill
\llap{\parbox[b]{\headwidth}{\raggedleft \PseudoHeadColor{#4}\strut}}}\headrule \vskip 2pt \ifPurposeText@{\HeadFont\HeadSize \ifodd\count\z@  \hbox to \headwidth{\hfill \purposetext}\else \hbox to \headwidth{\purposetext\hfill}  \fi} \else \vskip 10pt \fi}}#5
\ifPurposeText@
  \PseudoHead
  \global\def\PseudoHeadColor{\Gray}
  %\global\let\HeadFont=\PseudoHeadFont
  %\global\let\HeadSize=\PseudoHeadSize
  %\global\def\PseudoHeadColor##1{\Gray{##1}}
\fi
\global\PurposeText@false\global\def\purposetext{}}



\def\@fancyfoot#1#2#3#4#5{\ifodd\count\z@\hspace{-\marginpartotalwidth}\fi#1\hbox to\headwidth{\vbox{\footrule
\hbox{\rlap{\parbox[t]{\headwidth}{\raggedright#2\strut}}\hfill
\parbox[t]{\headwidth}{\centering#3\strut}\hfill
\llap{\parbox[t]{\headwidth}{\raggedleft#4\strut}}}}}#5}

\def\ps@fancy{
\let\@mkboth\markboth
\@ifundefined{chapter}{\def\sectionmark##1{\markboth
{\uppercase{\ifnum \c@secnumdepth>\z@
 \thesection\hskip 1em\relax \fi ##1}}{}}
\def\subsectionmark##1{\markright {\ifnum \c@secnumdepth >\@ne
 \thesubsection\hskip 1em\relax \fi ##1}}}
{\def\chaptermark##1{\markboth {\uppercase{\ifnum \c@secnumdepth>\m@ne
 \@chapapp\ \thechapter. \ \fi ##1}}{}}
\def\sectionmark##1{\markright{\uppercase{\ifnum \c@secnumdepth >\z@
 \thesection. \ \fi ##1}}}}
\def\@oddhead{\@fancyhead\relax\@olhead\@ochead\@orhead\hss}
\def\@oddfoot{\@fancyfoot\relax\@olfoot\@ocfoot\@orfoot\hss}
\def\@evenhead{\@fancyhead\hss\@elhead\@echead\@erhead\relax}
\def\@evenfoot{\@fancyfoot\hss\@elfoot\@ecfoot\@erfoot\relax}
\headwidth\textwidth}

% remember chapter and section titles in the right format
\renewcommand{\chaptermark}[1]{\markboth
{\chaptername\ \thechapter\ \ensuremath{\bullet}\ {#1}}
{} 
}
\renewcommand{\sectionmark}[1]{\markright{#1}}

%
% Occationally want more than one heading per chapter
%  (i.e. for man pages)  Add an additional variable
%  for the inside heading
\newcommand{\insideheading}{}


% set up the headers in the right fonts
% We want \rightmark on the outside corners on all pages but those that start
% a new chapter
\lhead[\fancyplain{}{\HeadFont\HeadSize\rightmark}]%
{\fancyplain{}{\insideheading}}
\rhead[\fancyplain{}{\insideheading}]%
{\fancyplain{}{\HeadFont\HeadSize\rightmark}}


% The footers
%
\lfoot[\PageFont\PageSize Printed in Canada]
      {\PageFont\PageSize Printed in Canada}
\rfoot[\PageFont\PageSize Page \thepage \ of \TotalPages]%
      {\PageFont\PageSize Page \thepage \ of \TotalPages}
 
\cfoot[\PageFont\PageSize \PartNum]%
      {\PageFont\PageSize \PartNum}

\renewcommand\tableofcontents{%
  % start the minitocs
  \dominitoc
    \if@twocolumn
      \@restonecoltrue\onecolumn
    \else
      \@restonecolfalse
    \fi
    \chapter*{\contentsname}
        %\@mkboth{%
           %\MakeUppercase\contentsname}{\MakeUppercase\contentsname}}%
    \@starttoc{toc}%
    \if@restonecol\twocolumn\fi
    }


%%%%
%
% A command to put text/images in the margin right justified...
%
%%%%

\newcommand\marginNote[1]{%
    \margin@note{#1}%
    \global\subsectskip@true%
    \def\par{\@restorepar\@par \global\subsectskip@false}
  }

\newcommand\margin@note[1]{
  {\mbox{}%
   \hskip -5.5pt%
   \marginpar{\raggedleft \hspace{0pt}#1}%
  }
}

%%%%
%
% Section headers, paragraph indenting, etc.
%
%%%%

% paras are not to be indented
\setlength{\parindent}{0pt}
\parskip 9pt

% no numbering of sections
\setcounter{secnumdepth}{0}

\newcommand\secstyle[1]{#1}


% \@startsection{name}{depth}{indent}{beforeskip}{afterskip}{style}

% to fix spacing after a subsection/subsection if the next thing is an alltt. etc.
\newif\ifsubsectskip@
\subsectskip@false

% for subsection no indenting and no after skip; placement and style are governed
% by \seubsectionstyle below
% change the rubber length beforeskip to adjust spacing *before* subsection
\renewcommand{\subsection}{\@restorepar\global\subsectskip@true \@startsection{subsection}{2}{0pt}{-1.25ex plus -1ex minus -.2ex}{0pt}{}}

% ditto for subsubsection
\renewcommand{\subsubsection}{\@restorepar\global\subsectskip@true \@startsection{subsubsection}{3}{0pt}{-1.25ex plus -1ex minus -.2ex}{-0pt}{}}

\newcommand\sectionstyle[1]{\SFont\Ssize #1}
\newcommand\subsectionstyle[1]{\marginNote{\SSFont\SSsize #1}\hskip 0pt}
\newcommand\subsubsectionstyle[1]{\marginNote{\SSSFont\SSSsize #1}\hskip 0pt}

%
% pseudo section
%
\newif\ifNoPseudoSection@
\NoPseudoSection@true

\newcommand{\PseudoSection}[2]
{
  % \clearpage
  % \global\let\HeadFont=\MainHeadFont
  % \global\let\HeadSize=\MainHeadSize
  \PseudoHead
  \NoPseudoSection@false
  \section{#1}
  % \sectionmark{\Gray{#1}}
  \Purpose{#2}
}

\newcommand{\PseudoSectionTitled}[3]
{
  % \global\let\HeadFont=\MainHeadFont
  % \global\let\HeadSize=\MainHeadSize
  \PseudoHead
  \NoPseudoSection@false
  \section[#1]{#2}
  % \sectionmark{\Gray{#1}}
  \Purpose{#3}
}

% change \@sect so that section headings are printed using our styles

% min space left on a page to start a new section
\def\minbreak{5\baselineskip}
\newdimen\temppakg

\def\@sect#1#2#3#4#5#6[#7]#8{%
  \temppakg=\pagegoal
  \advance\temppakg-\pagetotal
  \ifdim\temppakg<\minbreak\newpage\fi

  \ifnum #2>\c@secnumdepth
  \let\@svsec\@empty\else
  \refstepcounter{#1}%
  \protected@edef\@svsec{\@seccntformat{#1}\relax}%
  \fi
  \@tempskipa #5\relax
  \ifdim \@tempskipa>\z@
  \ifNoPseudoSection@
    \begingroup #6\relax
    \@hangfrom{\hskip #3\relax\@svsec}%
    {\interlinepenalty \@M \csname #1style\endcsname{#8}\par}%
    \endgroup
  \fi
  \csname #1mark\endcsname{#8}\addcontentsline
  {toc}{#1}{\ifnum #2>\c@secnumdepth \else
    \protect\numberline{\csname the#1\endcsname}\fi
    #7}\else
  \def\@svsechd{\ifNoPseudoSection@#6\hskip #3\relax  %% \relax added 2 May 90
    \@svsec \csname #1style\endcsname{#8}\fi\csname #1mark\endcsname
    {#8}\addcontentsline
    {toc}{#1}{\ifnum #2>\c@secnumdepth \else
      \protect\numberline{\csname the#1\endcsname}%
      \fi
      #7}}\fi
  \@xsect{#5}
  \NoPseudoSection@true
  % \vskip -.75\baselineskip
  \def\par{\@restorepar\@par \global\subsectskip@false}
}

%
% The following is needed to make the section* commands
% do the same thing as the section command...
%
% From chrish:
%
% 5) \chapter*, \section*, \subsection*... formatted the same as \chapter,
%   \section, and \subsection, but don't make entries in the TOC; should
%   still change the headers/footers though


\def\@startsection#1#2#3#4#5#6{\if@noskipsec \leavevmode \fi
   \par \@tempskipa #4\relax
   \@afterindenttrue
   \ifdim \@tempskipa <\z@
     \@tempskipa -\@tempskipa \@afterindentfalse
   \fi
   \if@nobreak \everypar{}\else
     \addpenalty\@secpenalty\addvspace\@tempskipa
   \fi
   \@ifstar
     {\@ssect{#3}{#4}{#5}{#6}{#1}}%
     {\@dblarg{\@sect{#1}{#2}{#3}{#4}{#5}{#6}}}}

\def\@ssect#1#2#3#4#5#6{%
   \temppakg=\pagegoal
   \advance\temppakg-\pagetotal
   \ifdim\temppakg<\minbreak\newpage\fi
   \@tempskipa #3\relax
   \ifdim \@tempskipa>\z@
     \begingroup #4\@hangfrom{\hskip #1}{\interlinepenalty \@M \csname #5style\endcsname {#6}\csname #5mark\endcsname{#6}\par}%
     \endgroup
   \else \def\@svsechd{#4\hskip #1\relax \csname #5style\endcsname {#6} \csname #5mark\endcsname{#6} }\fi
    \@xsect{#3} \def\par{\@restorepar\@par \global\subsectskip@false}}

\newcommand{\schaptermark}[1]{\markboth{#1}{}}

%%%%
%
% First page of a chapter
%
% . clear double pages so chapters start on odd pages
% . formatting chapater heading, etc.
% . invoke minitoc
%
%%%%

\newif\ifDoMinitoc@
\DoMinitoc@true

\newcommand{\ChapterNoToc}{
  \global\DoMinitoc@false
  \chapter
}

%%%
%
%  Define the registered trade mark symbol
%
%%%
%\def\rtm{\smaller{\raisebox{2ex}{\smaller{\Pisymbol{psy}{226}}}}}
%\def\TM{\smaller{\raisebox{2ex}{\smaller{\Pisymbol{psy}{228}}}}}
\def\rtm{\raisebox{.5ex}{\Pisymbol{psy}{226}}}
\def\TM{\raisebox{.5ex}{\Pisymbol{psy}{228}}}

%%%
%
%  Title page   \MakeTitle{BookSet}{BookTitle}
%          or	\MakeTitleCmt{BookSet}{BookTitle}{BookComment}
%
%%%
\def\MakeTitleCmt#1#2#3{\MainHead%
    \thispagestyle{empty}
    % format book  heading
    \vspace*{54\p@}%
    {\parindent \z@ \raggedleft
      \TitleMainFont \TitleMainSize \slshape 
      \moveleft\marginpartotalwidth\hbox to\headwidth{\vbox{\hbox
          {\rlap\hfill
            \parbox[b]{\headwidth}{\centering\strut}\hfill
            \llap{\parbox[b]{\headwidth}%
                            {\raggedleft #1\strut}%
                 }
          }\multiply\headrulewidth\tw@%
           \multiply\headrulewidth\tw@%
           \headrule%
           \divide\headrulewidth\tw@%
           \divide\headrulewidth\tw@%
      }}\relax
      \vskip -10pt{\raggedleft \TitleSubFont \TitleSubSize #2\par}
      \vskip 5cm{\raggedleft \TitleCmtFont \TitleCmtSize #3\par}
    }
}

\def\MakeTitle#1#2{\MakeTitleCmt{#1}{#2}{}}


\renewcommand\@makechapterhead[1]{%
  {
    % start chapters on odd side
    \cleardoublepage
    %
    % reset the Heading font to the standard case, just incase
    % things were changed via a PseudoSection{}  [bobbyc]
      % \global\let\HeadFont=\MainHeadFont
      % \global\let\HeadSize=\MainHeadSize
    \MainHead
    % format chapter heading
    \vspace*{60\p@}%
    {\parindent \z@ \raggedleft
      \ChFont \ChSize \slshape 
      \moveleft\marginpartotalwidth\hbox to\headwidth{\vbox{\hbox
          {\rlap\hfill
            \parbox[b]{\headwidth}{\centering\strut}\hfill
            \llap{\parbox[b]{\headwidth}%
                            {\raggedleft\@chapapp\ \thechapter\strut}%
                 }
          }\multiply\headrulewidth\tw@%
           \headrule%
           \divide\headrulewidth\tw@%
      }}\relax
      \vskip -10pt{\raggedleft \ChFont \ChSize \upshape#1\par\nobreak}%
      \par
    }
    \vspace{30pt}
    \ifDoMinitoc@
      % print heading for chapter table of contents
      % and generate the chapter TOC
      % print heading for chapter table of contents
      % and generate the chapter TOC
      {\ChFont \ChSize \slshape \MiniTocTitle}%
      \vskip -18pt%
      \minitoc\clearpage%
      \renewcommand{\MiniTocTitle}{In this Chapter...}%
    \else
      % print heading to replace minitoc
      {\mtcSfont \NoMiniTocTitle}
      \renewcommand{\NoMiniTocTitle}{}
    \fi
    \global\DoMinitoc@true
%                       I think this should be outside the \DoMintoc
%                       [BobbyC], but I'll leave it Anil's way.
%    \clearpage
    }
  }

\renewcommand\@makeschapterhead[1]{%
  {
    % start chapters on odd side
    \cleardoublepage
    %
    % reset the Heading font to the standard case, just incase
    % things were changed via a PseudoSection{}  [bobbyc]
      % \global\let\HeadFont=\MainHeadFont
      % \global\let\HeadSize=\MainHeadSize
    \MainHead
    %
    % format chapter heading
    \vspace*{60\p@}%
    {\parindent \z@ \raggedleft
      \ChFont \ChSize \slshape 
      \moveleft\marginpartotalwidth\hbox to\headwidth{\vbox{\hbox
          {\rlap\hfill
            \parbox[b]{\headwidth}{\centering\strut}\hfill
            \llap{\parbox[b]{\headwidth}%
                            {\raggedleft #1\strut}%
                 }
          }\multiply\headrulewidth\tw@%
           \headrule%
           \divide\headrulewidth\tw@%
      }}\relax
      % \vskip -10pt {\raggedleft \ChFont \ChSize \upshape#1\par\nobreak}
     \par
    }
    % want \chapter* to adjust the header marks also
    \schaptermark{#1}
    }
  }

%%%
%
% Adjust tables of contents
%
%%%


% no dotted lines
% no page numbers

\renewcommand\@dottedtocline[5]{\ifnum #1>\c@tocdepth \else
  \vskip \z@ \@plus.2\p@
  {\leftskip #2\relax %\rightskip % \@tocrmarg \parfillskip -\rightskip
    \parindent #2\relax\@afterindenttrue
   \interlinepenalty\@M
   \leavevmode
   \@tempdima #3\relax
   \advance\leftskip \@tempdima \null\hskip -\leftskip
%TOC lines that are not chapre lines...
%    spacing from pnumwidth
    {\TOCFont \TOCSize#4 \hbox to\@pnumwidth{\hss #5}}
      % Comment out section to put dots between the
      %   <<sectionname>> .... and ...  <pagenumber>
      %
      %\nobreak\leaders\hbox{$\m@th
      %\mkern \@dotsep mu\hbox{.}\mkern \@dotsep mu$
      %}\hskip 0pt \nobreak
           %\hb@xt@\@pnumwidth{%
            % \normalfont \normalcolor#5}
\par\vskip -0.5\baselineskip%
}\fi}
%TOC \par specififies spacing between one section and the next
%    \vskip -0.5\baselineskip


% The chapter in table of contents
%    - want 'Chapter NN' in margin
%

%TOC defines the chapter heading
%   call \numberline with \@chapapp
%
\def\@chapter[#1]#2{\ifnum \c@secnumdepth >\m@ne
                         \refstepcounter{chapter}%
                         \typeout{\@chapapp\space\thechapter.}%
                         \addcontentsline{toc}{chapter}%
                                   {\protect\numberline{\@chapapp \hbox{ } \thechapter}#1}%
                    \else
                      \addcontentsline{toc}{chapter}{#1}%
                    \fi
                    \chaptermark{#1}%
                    \addtocontents{lof}{\protect\addvspace{10\p@}}%
                    \addtocontents{lot}{\protect\addvspace{10\p@}}%
                    \if@twocolumn
                      \@topnewpage[\@makechapterhead{#2}]%
                    \else
                      \@makechapterhead{#2}%
                      \@afterheading
                    \fi}

\let\mtc@svchapter\@chapter %
\def\@chapter{\stepcounter{mtc}\mtc@svchapter} %

%    Style of the TOC ``Chapter'' heading.
%
%  WAS:
%\def\numberline#1{\marginNote{\TOCHeadFont \TOCHeadSize \slshape \chaptername \hbox{ } #1}}
% but modified so that ``Appendix'' will get printed.
% This essentially puts the words ``Chapter'' or ``Appendix''
% into the TOC.
%
\def\numberline#1{\marginNote{\TOCHeadFont \TOCHeadSize \slshape #1}}

\def\TOCentry#1{%
   \addtocontents{toc}{\protect {%
      \TOCHeadFont\TOCHeadSize%
      ~\hspace{-.12em}#1  \nobreak \nobreak\hbox to \@pnumwidth{\hss \thepage}%
      \par%
      \vskip -0.5\baselineskip %
   }}
}

% The chapter in table of contents
%    - want 'Chapter NN' in margin
%    - don't want page numbers
%

\renewcommand*\l@chapter[2]{%
  \ifnum \c@tocdepth >\m@ne
    \addpenalty{-\@highpenalty}%
    \vskip 1.0em \@plus\p@
    \setlength\@tempdima{1.5em}%
    \begingroup
      \parindent \z@ \rightskip \@pnumwidth
      %\parfillskip -\@pnumwidth
%TOC defines the font for the heading
      \leavevmode \TOCHeadFont\TOCHeadSize
      {\raggedright
      \advance\leftskip\@tempdima
      \hskip -\leftskip
%     #1\nobreak \nobreak\hbox to \@pnumwidth {\hbox{} #2}\par
      #1\nobreak \nobreak\hbox to \@pnumwidth{\hss #2}\par%
      }
%TOC \par changes the spaceing between the chapter and the 
%    next section  (Can add \vskip -0.5\baselineskip )
      \vskip -0.5\baselineskip
      \penalty\@highpenalty
    \endgroup
  \fi}


%%%%
%
% Redefine the tabbing of sections/subsections, etc.
%
%%%%
\renewcommand*\l@section{\@dottedtocline{1}{.25em}{2.3em}}
\renewcommand*\l@subsection{\@dottedtocline{2}{1.8em}{3.2em}}
\renewcommand*\l@subsubsection{\@dottedtocline{3}{3.8em}{4.1em}}
\renewcommand*\l@paragraph{\@dottedtocline{4}{7.0em}{5em}}
\renewcommand*\l@subparagraph{\@dottedtocline{5}{10em}{6em}}


%%%%
%
% Redfine \ttfamily
%
%%%%

\DeclareFontFamily{OT1}{pcrb}{\hyphenchar \font =-1}

\DeclareFontShape{OT1}{pcrb}{b}{n}{
   <-> pcrr7t
}{}

\DeclareFontShape{OT1}{pcrb}{b}{sc}{
   <-> pcrrc7t
}{}

\DeclareFontShape{OT1}{pcrb}{b}{sl}{
   <-> pcrro7t
}{}

\DeclareFontShape{OT1}{pcrb}{m}{n}{
   <-> pcrb7t
}{}

\DeclareFontShape{OT1}{pcrb}{m}{sc}{
   <-> pcrbc7t
}{}

\DeclareFontShape{OT1}{pcrb}{m}{sl}{
   <-> pcrbo7t
}{}


\DeclareFontShape{OT1}{pcrb}{bx}{n}{<->ssub * pcr/m/n}{}
\DeclareFontShape{OT1}{pcrb}{bx}{sc}{<->ssub * pcr/m/sc}{}
\DeclareFontShape{OT1}{pcrb}{bx}{sl}{<->ssub * pcr/m/sl}{}
\DeclareFontShape{OT1}{pcrb}{b}{it}{<->ssub * pcr/m/sl}{}
\DeclareFontShape{OT1}{pcrb}{bx}{it}{<->ssub * pcr/m/it}{}
\DeclareFontShape{OT1}{pcrb}{l}{n}{<->ssub * pcr/b/n}{}
\DeclareFontShape{OT1}{pcrb}{l}{sc}{<->ssub * pcr/b/sc}{}
\DeclareFontShape{OT1}{pcrb}{l}{sl}{<->ssub * pcr/b/sl}{}
\DeclareFontShape{OT1}{pcrb}{m}{it}{<->ssub * pcr/b/sl}{}
\DeclareFontShape{OT1}{pcrb}{l}{it}{<->ssub * pcr/b/it}{}

\renewcommand{\ttdefault}{pcrb}

\DeclareFontFamily{OMS}{pcrb}{\skewchar\font'60}
\DeclareFontShape{OMS}{pcrb}{b}{n}
   {<-> ssub * cmsy/m/n}{}
\DeclareFontShape{OMS}{pcrb}{b}{it}
   {<-> ssub * cmsy/m/n}{}
\DeclareFontShape{OMS}{pcrb}{b}{sl}
   {<-> ssub * cmsy/m/n}{}
\DeclareFontShape{OMS}{pcrb}{b}{sc}
   {<-> ssub * cmsy/m/n}{}
\DeclareFontShape{OMS}{pcrb}{m}{n}
   {<-> ssub * cmsy/b/n}{}
\DeclareFontShape{OMS}{pcrb}{m}{it}
   {<-> ssub * cmsy/b/n}{}
\DeclareFontShape{OMS}{pcrb}{m}{sl}
   {<-> ssub * cmsy/b/n}{}
\DeclareFontShape{OMS}{pcrb}{m}{sc}
   {<-> ssub * cmsy/b/n}{}

%%%%
%
% Justification
%
%%%%

% left justified; no right justification
\raggedright
% don't fill vertically
\raggedbottom

%%%%
%
% Justification
%
%%%%

%
% at cleardoublepage we don't want the possibly skipped even page to
% be numbered (taken from ``ltoutput.dtx''):
\renewcommand{\cleardoublepage}{%
  \clearpage%
  \if@twoside \ifodd\c@page\else \thispagestyle{empty}
    \hbox{}\newpage\if@twocolumn\hbox{}\newpage\fi%
  \fi\fi%
}


%%%%
%
%   Captions
% *SIGH*
%
% The stuff on side caption does not work
% So, we will do something simple for now, i.e., use \marginNote
% for generating side captions. Or, use the \sidecaption command
% below outside the floats, as in:
% \sidecaption{this is a fake side caption}
% \begin{figure}
% ...
% \end{figure}
%
%%%%

\setlength{\abovecaptionskip}{4pt}
\setlength{\belowcaptionskip}{8pt}

\newif\if@sidecap
\@sidecapfalse

%% \newcommand\sidecaption{\@sidecaptrue}
%% \newcommand\nosidecaption{\@sidecapfalse}
\newcommand{\sidecaption}[1]{\margin@note{\MarginCapFont\MarginCapSize #1}\par}

\long\def\@makecaption#1#2{%
  \vskip\abovecaptionskip
  \hrule
  \sbox\@tempboxa{\CapFont\CapSize #2}%
  \ifdim \wd\@tempboxa >\hsize
     \if@sidecap
        \marginpar{\raggedleft\MarginCapFont\MarginCapSize #2}
     \else
       {\raggedright\CapFont\CapSize #2\par}
     \fi
\else
    \global \@minipagefalse
     \if@sidecap
        \marginpar{\raggedleft\MarginCapFont\MarginCapSize #2}
     \else
       \hb@xt@\hsize{\hfil\box\@tempboxa\hfil}%
     \fi
  \fi
  \vskip\belowcaptionskip}

\def\LT@mkcaption#1{%
  \LT@mc{\LT@cols}{c}{\hbox to \z@{\hss\parbox[t]{\LTcapwidth}{%
    \setbox\@tempboxa\hbox{\CapFont\CapSize  #1}%
   \ifdim \wd\@tempboxa >\hsize \CapFont\CapSize  #1
     \else \hbox to\hsize{\hfil\box\@tempboxa\hfil}\fi
            \endgraf\vskip\baselineskip}\hss}}}

%%%%
%
%  New environment for typesetting wide text 
%     the Margin becomes part of text
%
%  Usage:
%    \begin{widetext}
%       anything suitable in a minipage
%    \end{widetext}
%
%  Caveats:
%    . The page boundary must not be crossed inside the widetext
%    . Obviously, the \margiNote command is meaningless inside widetext
%    . whatever else I don't know
% 
%%%%

\newdimen\widetextwidth
\setlength{\widetextwidth}{\textwidth}
\addtolength{\widetextwidth}{\marginpartotalwidth}
\newenvironment{widetext}
  {% July 18, 1996:   Might want a
   %
   % \global\subsectskip@false
   %
   % or might need something even more complex to deal with wide
   % things right after a margin.  
   %
    \mbox{}\kern-\marginpartotalwidth
    \begin{minipage}{\widetextwidth}
    }
  {
    \end{minipage}
    }


%%%%
%
% ChrisH sez:
% 
% 9) a good table environment... we could use tabularx except it makes all
%    the auto-sized columns the same width, and it doesn't work for
%    multi-page tables.  We need:
% 
%       - auto-sized column widths with different widths for each column
%       - text wrapping inside the columns
%       - text is left-justified and \raggedright
%       - may need to specify right-justified or centred columns at some
%         point (don't need it now)
% 
%    tabularx is probably a good place to start, unless its disgusting and
%    can't be easily modified
% 
%%%%

%%%
%
% Another change that has been made to tabularx is to treat
% all 'X' columns like  'l' columns if the contents of the 
% 'X' columns can fit within the specified table width
%
%%%

% control interline spacing in X columns cells
%
\newcommand{\TabularXSize}{\@setfontsize\TabularXSize{11pt}{10pt}}
%
% control space beteen table rows
\newcommand{\TabularStretch}{1.5}    % control space between rows
\newcommand{\TabularXStretch}{0.95}  % control inter-line space within cells

% 
% we will use tabularx and to get multi-page tables let's make
% tabularx use longtable instead of the normal tabular
%

% code here now lives in longtabularx.sty

\def\tabularx#1{%
  % if a tabularx follows a subsection (which is typeset with a marginpar 
  % the following ensures that the tabularx starts with a \par
  %\hspace{0pt}\par%
    \if@minipage
          \hspace{0pt}\par%
    \else
      \ifsubsectskip@ \hspace{0pt}\par%
                      \vskip -1.9\baselineskip%
                      \global\subsectskip@false%
      \else
             \hspace{0pt}\par%
             \vskip -.7\baselineskip%
      \fi
    \fi
  %\ifnum \@listdepth >0 \setlength{\LTleft}{\@totalleftmargin}\fi
    \ifnum \@listdepth >0
      \ifdim \LTleft = -\marginpartotalwidth \else
         \setlength{\LTleft}{\@totalleftmargin}
      \fi
   \fi

  
  \renewcommand{\arraystretch}{\TabularStretch}
  \edef\TX@{\@currenvir}%
  {\ifnum0=`}\fi
  %\global\subsectskip@false
  \TX@target#1\relax
  \TX@typeout{Target width: #1 = \the\TX@target.}%
  \toks@{}\TX@get@body%
  }

\renewcommand{\LTX@spacing}{\TabularXStretch}

%
% If \TX@abort is true and the table contents are wider than 
% the maximum width, abort processing
%

\newif\ifTX@abort@
\TX@abort@true
\def\tabularxNoAbort{\global\TX@abort@false}

\def\TX@arith{%
  \TX@false
  \ifdim\TX@old@table=\wd\@tempboxa
    \TX@col@width=\TX@old@col
    \TX@typeout@{Reached minimum width, backing up.}%
  \else
    \dimen@=\wd\@tempboxa
    \advance\dimen@ -\TX@target
    \ifdim\dimen@<\TX@delta
      \TX@typeout@{Reached target.}%
    \else
      \ifnum\TX@cols>\@ne
        \advance\TX@cols by \m@ne
      \fi
      \divide\dimen@ by \TX@cols
      \advance\dimen@ by -\TX@col@width
      \ifdim \dimen@ > \z@
        \TX@typeout@{Columns too narrow.}%
        \ifTX@abort@
            \typeout{}
            \typeout{****************************************}
            \typeout{***** Table contents are too wide. *****}
            \typeout{***** Aborting...                  *****}
            \typeout{****************************************}
            \typeout{}
            \stop
         \fi
      \else
        \TX@old@col=\TX@col@width
        \TX@old@table=\wd\@tempboxa
        \TX@col@width=-\dimen@
        \TX@true
      \fi
    \fi
  \fi}


%
% want the tables left justified
%
\setlength{\LTleft}{0pt}
\setlength{\LTright}{\fill}

%%%%%%
%%
%%  Want all bullets in itemize to be the same
%%
%%      BobbyC
%%%%%%
\renewcommand{\labelitemii}{$\m@th\bullet$}
\renewcommand{\labelitemiii}{$\m@th\bullet$}
\renewcommand{\labelitemiv}{$\m@th\bullet$}

%%%%%
%%
%% Re-define the list environment to consider the 
%%            @subsectskip variable:
%%
%%%%%
\def\list#1#2{\ifnum \@listdepth >5\relax \@toodeep
     \else \global\advance\@listdepth\@ne \fi
  %
  \ifsubsectskip@ \hspace{0pt}%                  new
                  ~\vskip -3.27\baselineskip~%   new
                  \global\subsectskip@false%     new
  \fi%                                           new
  %
  \rightmargin \z@ \listparindent\z@ \itemindent\z@
  \csname @list\romannumeral\the\@listdepth\endcsname
  \def\@itemlabel{#1}\let\makelabel\@mklab \@nmbrlistfalse #2\relax
  \@trivlist
  \parskip\parsep \parindent\listparindent
  \advance\linewidth -\rightmargin \advance\linewidth -\leftmargin
  \advance\@totalleftmargin \leftmargin
  \parshape \@ne \@totalleftmargin \linewidth
  \ignorespaces}

%%%%%
%%
%% More stuff from The LaTeX Companion, page 63
%%
%%%%%
\newif\ifinentry@
\inentry@false

\newcommand{\entrylabel}[1]{\mbox{\raggedright{#1}}\hfil}
\newenvironment{entry}
	{\@restorepar\ifhmode \global\subsectskip@false \fi
	\begin{list}{}%
		{\renewcommand{\makelabel}{\entrylabel}%
		\setlength{\labelwidth}{35pt}%
		\setlength{\leftmargin}{\labelwidth+\labelsep}%
		\setlength{\@rightskip}{0pt plus 1fil}%
		}%
    }
	{\end{list}\global\inentry@false}
%%%
%
%  Define a varible length version of LMentry so that entries can
%  1)  resize to user specified value
%  2)  Wrap to the next line to fit extra-long lines
%  3)  Allow line-breaks
%
%  Bobbyc  June 19, 1996
%%%
\newlength{\SMARTlabelwidth}  % re-calculated for each \begin{SMARTentry}
                              % using the \CalcLabelWidth{} function

\newlength{\SMARTtmpLen}  % tmp variables for CalcLabelWidth...
\newlength{\SMARTmaxLen}  %

% \ZeroLabelWidth doesn't actually zero it, it sets the label width to
% .25in.
\def\ZeroLabelWidth{\setlength{\SMARTlabelwidth}{.25in}}
\ZeroLabelWidth

\def\CalcLabelWidth#1{%
  \settowidth{\SMARTtmpLen}{\hbox{%
                            \expandafter\tabular{l}#1
                            \endtabular}}
  \setlength{\SMARTmaxLen}{\textwidth / 4}%
  \ifthenelse{\lengthtest{\SMARTtmpLen < \SMARTmaxLen}}{%
     \ifthenelse{\lengthtest{\SMARTtmpLen > \SMARTlabelwidth}}%
        {\setlength{\SMARTlabelwidth}{\SMARTtmpLen}}{}
  }{}
}


\newenvironment{SMARTlist}
	{\@restorepar\ifhmode \global\subsectskip@false \fi
	\begin{list}{}%
		{\renewcommand{\makelabel}{\entrylabel}%
		\setlength{\labelwidth}{\SMARTlabelwidth}%
%%%
%               I was working late one Saturday night, and suddenly
%               The list needed another 3points to align
%               with the previous paragraph (Don't quite
%               know why....)
%
%               Bobby (June 22, 1996)
%	was:	\setlength{\leftmargin}{\labelwidth+\labelsep}%
%%%
		\setlength{\leftmargin}{\labelwidth+\labelsep + 3pt}%
                \setlength{\@rightskip}{0pt plus 1fil}%
		}%
    }
    {\end{list}%
    }


\newbox\SMARTlabelbox
\newcommand{\SMARTentrylabel}[1]{%
  \setbox\SMARTlabelbox=\hbox{%
    \expandafter\tabular{l}#1
    \endtabular}
  {% align the left of the label box with preceding text
   \hskip -.75em
   \ifdim\wd\SMARTlabelbox>\linewidth%
      \parbox[b]{\linewidth + \labelwidth + \labelsep}
                {#1}%   This doesn't work --> \vspace{\baselineskip / 2}}
   \else
      % set the label in a parbox of width \labelwidth
      \parbox[b]{\labelwidth}{\makebox[0pt][l]{
          \expandafter\tabular{l}
          #1\\
          \endtabular}
      % if the label is wide skip a line so the para starts on next line
      \ifdim\wd\SMARTlabelbox>\labelwidth %
         \vskip \baselineskip%
      \else
         %
         % even if the label is narrow but is multi-line, align the
         % para with the basline of the last line of the label
         \ifdim\ht\SMARTlabelbox>2ex \vskip -0.4\ht\strutbox \fi
      \fi
      }
  \fi 
  }%
  \hfil\relax%
}


\newenvironment{SMARTentry}
  {%
     \renewcommand{\entrylabel}{\SMARTentrylabel}%
     % 
     % I was hoping the test below would not be necessary,
     % because I put the logic into the 'list' definition...
     % ...but... it seems to want a different \baselineskip
     %   [BobbyC]  [ 7/17/96]
     %
     \ifsubsectskip@ \hspace{0pt}%
                     ~\vskip -3.75\baselineskip~%
                     \global\subsectskip@false%
     \fi%
     \begin{SMARTlist}
     \setlength{\tabcolsep}{0pt}
     \setlength{\itemindent}{0em}
     \global\inentry@true%
  }%
  {\end{SMARTlist}%
   \global\inentry@false
  }

%%%%%
%%  Entry environment that automatically skips to the
%%  next line if entry is too long.
%%
%%  From:  "The Latex Companion", page 64
%%
%%  BobbyC
%%%%%
\newlength{\Lentrylen}
\newcommand{\Lentrylabel}[1]{%

  \settowidth{\Lentrylen}{#1}%
  \ifthenelse{\lengthtest{\Lentrylen > \labelwidth}}%
     {\parbox[b]{\labelwidth}%               term > labelwidth
        {\makebox[0pt][l]{#1}\\}}%
     {#1}%                         term < labelwidth
  \hfil\relax}%

\newenvironment{Lentry}
  {\renewcommand{\entrylabel}{\Lentrylabel}%
   \begin{entry}
     }%
  {\end{entry}}

%%%%%
%% Entry environment that allows multi-line labels
%%
%%  From:  "The Latex Companion", page 65
%%
%%  BobbyC
%%%%%
\newcommand{\Mentrylabel}[1]%
   {     \raisebox{0pt}[1ex][0pt]{\makebox[\labelwidth][l]%
       {\parbox[t]{\labelwidth}{\hspace{0pt}\raggedright{#1}}}}}
\newenvironment{Mentry}%
   {\renewcommand{\entrylabel}{\Mentrylabel}\begin{entry}
     }%
   {\end{entry}}


%%%%%
%%  Hack of the ``Notes'' environment to make lists behave
%%
%%  From:  "The Latex Companion", page 65
%%
%%  BobbyC
%%%%%
\newcounter{notes}
\renewenvironment{itemize}
  {\begin{list}{$\m@th\bullet$ \hfill}{\usecounter{notes}%
         \setlength{\labelsep}{0pt}\setlength{\leftmargin}{1.25em}%
         \setlength{\labelwidth}{1.25em}%
         \setlength{\listparindent}{0pt}%
	 \setlength{\topsep}{0pt}
         }}
  {\end{list}}

\newenvironment{plain_ul}
  {\begin{list}{}{\usecounter{notes}%
         \setlength{\labelsep}{0pt}\setlength{\leftmargin}{18pt}
         \setlength{\labelwidth}{18pt}
         \setlength{\listparindent}{0pt}%
         \setlength{\itemsep}{-0.5 \parsep}%
%         \setlength{\itemindent}{0pt}
         }}
  {\end{list}}

\newenvironment{interior_ul}
  {\begin{list}{- \hfill}{\usecounter{notes}%
         \setlength{\labelsep}{0pt}\setlength{\leftmargin}{1.25em}%
         \setlength{\labelwidth}{1.25em}%
         \setlength{\listparindent}{0pt}%
	 \setlength{\topsep}{0pt}
         }}
  {\end{list}}

% 
% The above is not the way to go; see below. -Anil
%

%
% define the way labels are numbered
\renewcommand\theenumi{\@arabic\c@enumi}
\renewcommand\theenumii{\theenumi\@alph\c@enumii}
\renewcommand\theenumiii{\theenumii.\@roman\c@enumiii}
\renewcommand\theenumiv{\theenumiii(\@arabic\c@enumiv)}
%
% define the way labels are printed
\renewcommand\labelenumi{\sfnfamily \theenumi}
\renewcommand\labelenumii{\sfnfamily \theenumii}
\renewcommand\labelenumiii{\sfnfamily \theenumiii}
\renewcommand\labelenumiv{\sfnfamily \theenumiv}
%
%
% want labels left justified;
% redefine the \makelabel in LaTeX's definition of \enumerate
\def\enumerate{%
  \ifnum \@enumdepth >\thr@@\@toodeep\else
    \advance\@enumdepth\@ne
    \edef\@enumctr{enum\romannumeral\the\@enumdepth}%
      \list
        {\csname label\@enumctr\endcsname}%
        {\usecounter\@enumctr
	  \setlength{\topsep}{0pt}
	  \def\makelabel##1{\rlap{##1}\hss}%
        }%
  \fi}

%%%%%
%% Entry environment that allows multi-line labels
%% AND follows the Lentry logic
%%
%%%%%
\newbox\LMlabelbox
%%\newcommand{\LMentrylabel}[1]{%
%%  \setbox\LMlabelbox=\hbox{%
%%    \expandafter\tabular{l}#1
%%    \endtabular}
%%  {\parbox[b]{\labelwidth}% 
%%    {\makebox[0pt][l]{
%%        \expandafter\tabular{l}
%%        #1
%%        % The following is ugly (but functional) and should be looked at when time permits
%%        %    - Anil
%%        \ifdim\wd\LMlabelbox>\labelwidth \\~ \else
%%           \ifdim\ht\LMlabelbox>2ex \\~ \fi
%%        \fi
%%        \endtabular}
%%      \ifdim\wd\LMlabelbox>\labelwidth \vskip 0pt
%%      \fi
%%      }% makebox{}{}
%%    }% \parbox[b]{}
%%  \hfil\relax%
%%}
%%
\newcommand{\LMentrylabel}[1]{%
  \setbox\LMlabelbox=\hbox{%
    \expandafter\tabular{l}#1
    \endtabular}
  {    % align the left of the label box with preceding text
    \hskip -.75em
    % set the label in a parbox of width \labelwidth
    \parbox[b]{\labelwidth}{\makebox[0pt][l]{
          \expandafter\tabular{l}
          #1\\
          \endtabular}
      % if the label is wide skip a line so the para starts on next line
      \ifdim\wd\LMlabelbox>\labelwidth \vskip \baselineskip \else
         % even if the label is narrow but is multi-line, align the
         % para with the basline of the last line of the label
         \ifdim\ht\LMlabelbox>2ex \vskip -0.4\ht\strutbox \fi
      \fi
      }
    }%
  \hfil\relax%
}



\newenvironment{LMentry}
  {\renewcommand{\entrylabel}{\LMentrylabel}%
   \begin{entry}         
     % \setlength{\listparindent}{}%
     \setlength{\tabcolsep}{0pt}
     \setlength{\itemindent}{0em}
    }%
  {\end{entry}}

%%%%
%  Define a captioned figure environment.
%
%  BobbyC
%%%%
\def\CFigure#1#2%  
  {\parbox{\linewidth}{\raggedright\sizedGfx{#1}\vskip 0.5ex
		       \edef\@QNXtempa{}%
	               \edef\@QNXtempb{#2}%
		       \ifx \@QNXtempa \@QNXtempb  \vskip -.5\baselineskip
                       \else   \rule{\linewidth}{0.4 pt}
                               {\CapFont\CapSize{\raggedleft #2}}
			       \vskip .8\baselineskip%
		       \fi
                      }
   \leavevmode%
  }% Cfigure

%%%%
%  Define a widecaptioned Figure environment.
%
%  BobbyC
%%%%
\def\wideCFigure#1#2%  
  {\begin{widetext}%
   \parbox{\widetextwidth}%
          {\sizedGfx{#1}\vskip 0.5ex
           \hspace*{\marginpartotalwidth}\nobreak%
           \parbox{4.249999in}%
                  {\rule{4.249999in}{0.4 pt}
                       {\CapFont\CapSize\raggedright{#2}}
                  }
          }
           \end{widetext}
  }% wideCfigure


%%%%
%  Define a widetable environment.
%  Usage:  \begin{widetable}
%          \begin{tabularx}{\linewidth}{l>{\raggedright\arraybackslash}X}
%              :
%              :
%          \end{tabularx}
%          \end{widetable}
%
%  BobbyC
%%%%
\newenvironment{widetable}
  { \if@minipage
          \hspace{0pt}\par%
    \else
      \ifsubsectskip@ \hspace{0pt}\par%
                      \vskip -.2\baselineskip%
                      \global\subsectskip@false%
      \fi
    \fi

   \setlength{\LTleft}{-\marginpartotalwidth}%
  }
  {\setlength{\LTleft}{0pt}%
   \leavevmode%
  }

%
%  Redefine definition of "theindex" to space things
%  better and to allow the description at the top of
%  the page to be modified.
%
%         [BobbyC]
%
\newcommand\IndexDescription\indexname
\renewenvironment{theindex}
               {\if@twocolumn
                  \@restonecolfalse
                \else
                  \@restonecoltrue
                \fi
                \columnseprule \z@
                \columnsep 35\p@
                \twocolumn[\@makeschapterhead{\indexname}\vspace{1in}]%
                \@mkboth{\indexname}%
                        {\IndexDescription}%
                \thispagestyle{plain}\parindent\z@
                \parskip\z@ \@plus .3\p@\relax
                \let\item\@idxitem}
               {\if@restonecol\onecolumn\else\clearpage\fi}



%
% remove parskip at beginning
%

\renewenvironment{alltt}{%
  % \ifhmode \global\subsectskip@false \fi
  \ifsubsectskip@ \fi\def\par{\@restorepar\@par}
  \trivlist
  \item\relax
    \if@minipage
    \else
      \ifsubsectskip@ \vskip -1.6\baselineskip \global\subsectskip@false \fi
    \fi
    \leftskip\@totalleftmargin
    \rightskip\z@skip
    \parindent\z@
    \parfillskip\@flushglue
    \parskip\z@skip
    \@@par
    \@tempswafalse
    \def\par{%
      \if@tempswa
        \leavevmode\null\@@par\penalty\interlinepenalty
    \else
      \@tempswatrue
      \ifhmode\@@par\penalty\interlinepenalty\fi
    \fi}
    \obeylines
    \verbatim@font
    \let\org@prime~%
    \@noligs
    \everymath\expandafter{\the\everymath\let~\org@prime}
    \everydisplay\expandafter{\the\everydisplay\let~\org@prime}
    \let\org@dospecials\dospecials
    \g@remfrom@specials{\\}
    \g@remfrom@specials{\{}
    \g@remfrom@specials{\}}
    \let\do\@makeother
    \dospecials
    \let\dospecials\org@dospecials
    \frenchspacing\@vobeyspaces
    \everypar \expandafter{\the\everypar \unpenalty}
  }
  {\endtrivlist%
   \leavevmode%
   \vskip -1.1\baselineskip
  }


%
%
%  \keeptogether{LaTeX for some icon}{Some text}
%
%  icon is put in the margin, text is surrounded by rulers
%  the whole thing is kept on one page
%
%
\newbox\keepbox
\newlength\keepheight
\newbox\keeptmpbox
\newdimen\keepwidth
\def\keeptext{}
\newenvironment{QNXnote}[1]{
  \leavevmode%
  \global\def\keeptext{#1}\leavevmode\vskip -1.2\baselineskip
  \setbox\keepbox=\vbox\bgroup\parindent\z@\ignorespaces
  \mbox{}\newline\rule{\textwidth}{0.4 pt}\nopagebreak\\[0pt]\nopagebreak
  }
{   \egroup
  \setlength{\keepheight}{\ht\keepbox}
  \setlength{\keepwidth}{\marginparwidth}
  \addtolength{\keepwidth}{2.1pt}
  \setbox\keeptmpbox=\hbox to \keepwidth {
    \parbox[b]{ \marginparwidth}{\raggedleft \ignorespaces\hspace{0pt}\ignorespaces\keeptext}}
  \ifdim\keepheight<\ht\keeptmpbox
    \setlength{\keepheight}{\ht\keeptmpbox}
  \fi
  \ifdim\pagetotal<\pagegoal
    \tempakg=\pagegoal
    \advance\tempakg-\pagetotal
    \advance\tempakg-\baselineskip
    \ifdim\tempakg<\keepheight\newpage\vskip-\baselineskip\fi
  \fi
  \vbox  {\vbox to  \keepheight{\box\keepbox} \vskip -\baselineskip \vskip 2pt \rule{\textwidth}{0.4 pt}}
  \nopagebreak \margin@note{\vskip -\keepheight \ifinentry@ \vskip -0.3\baselineskip \else \vskip -1.3\baselineskip \fi \keeptext}\leavevmode\vskip -3\baselineskip \mbox{}
  \global\def\keeptext{}
  \leavevmode
}

%%%%%
% Cause Latex to break a line before, so that a
% widow has a companion at the beginning of a new page.
%
% See page 104 in the TeXbook for details.
%
% BobbyC
%
%%%%%
\widowpenalty=10000
\hyphenpenalty=10000


%%%
%  Define macros to print special characters
%  [BobbyC]  June 18, 1996
%%%
\chardef\OpenSquare=`\[
\chardef\CloseSquare=`\]
\chardef\Hat=`\^
\chardef\Tilde=`\~
\def\BackSlash{\protect\B@ckSlash}%
\def\B@ckSlash{\edef\@QNXtempa{\ttdefault}%
               \edef\@QNXtempb{\f@family}%
	       \ifmmode \delimiter''26E30F%
               \else \ifx \@QNXtempa \@QNXtempb \char`\\%
                     \else \ensuremath{\backslash}%
                     \fi%
               \fi}%

%%%
% Then there are a few more complex ones...
%%%
\chardef\OpenBraceC=`\{
\chardef\CloseBraceC=`\}
\chardef\GreaterThanC="3E % prints and upside-down ?
\chardef\LessThanC="3C    % prints and upside-down !
\chardef\VerticalBarC="7C % prints an horizontal line (sigh...)
\def\GreaterThan{\texttt{\GreaterThanC}}
\def\LessThan{\texttt{\LessThanC}}
\def\VerticalBar{\texttt{\VerticalBarC}}
\def\OpenBrace{\texttt{\OpenBraceC}}
\def\CloseBrace{\texttt{\CloseBraceC}}
\def\AllttUnderscore{\leavevmode \kern.1em \vbox{\hrule width.5em height.2ex}}
\def\ItDollar{\protect{\normalfont\char36}}

%%%
%  Define macros to print a nice back-quote..
%  [BobbyC]  July 4, 1996
%%%
\newcommand{\cttdefault}{cmtt}
\DeclareRobustCommand\cttfamily
        {\not@math@alphabet\cttfamily\mathtt
         \fontfamily\cttdefault\selectfont}
\DeclareTextFontCommand{\textctt}{\cttfamily}
\def\AllttBackQuote{\textctt `}
\def\AllttDash{\larger\larger\textctt -}

%%%%
%  Silly stuff for hacking the alltt environments...
%  [chrish] July 4, 1996
%%%%
\def\setSmallTT{\def\verbatim@font{\normalfont\ttfamily\ttSmallSize%
                   \let\do\do@noligs%
                   \verbatim@nolig@list}}

\def\setMediumTT{\def\verbatim@font{\normalfont\ttfamily\ttMediumSize%
                   \let\do\do@noligs%
                   \verbatim@nolig@list}}

\def\setLargeTT{\def\verbatim@font{\normalfont\ttfamily\ttLargeSize%
                   \let\do\do@noligs%
                   \verbatim@nolig@list}}


%%%
%
%  Better versions of \smaller, and \larger
%
%  Also removed the ``relsize'' package.
%%%
\def\smaller{%
\@tempdimb=\z@%
\advance\@tempdimb\f@size\p@%
%
   \advance\@tempdimb by -1pt%
   \fontsize{\@tempdimb}{\f@baselineskip\p@}\selectfont
}

\def\larger{%
\@tempdimb=\z@%
\advance\@tempdimb\f@size\p@%
%
   \advance\@tempdimb by 1pt%
   \fontsize{\@tempdimb}{\f@baselineskip\p@}\selectfont%
}


\def\text@tt#1{\edef\@QNXtempa{\rmdefault}%
               \edef\@QNXtempb{\f@family}%
               \edef\@QNXtempc{\sfdefault}%
               \ifx \@QNXtempa \@QNXtempb {\smaller\normalfont\texttt{#1}}%%
               \else \ifx \@QNXtempb \@QNXtempc {\larger\normalfont\texttt{#1}}%%
                     \else {\normalfont\texttt{#1}}%
                     \fi%
               \fi}%
\def\tthack#1{\protect\text@tt{#1}}%

%%%%%
% Extras for the top of the first page.
%%%%%

\def\Qtitle#1{\sffamily\bfseries%
              \@setfontsize\QtitleSize{24pt}{26pt}%
              \rightline{#1}%
              \normalfont\normalsize}

\def\Qsubtitle#1{\sffamily\bfseries%
              \@setfontsize\QsubtitleSize{18pt}{16pt}%
              \rightline{#1}%
              \normalfont\normalsize}

\def\Qnotetype#1{\sffamily\bfseries\itshape%
              \@setfontsize\QtitleSize{24pt}{26pt}%
              \rightline{#1}%
              \normalfont\normalsize}
